# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import collections
import ipaddress
import re
from itertools import tee

from azure.cli.core.commands.client_factory import get_mgmt_service_client
from azure.cli.core.commands.validators import get_default_location_from_resource_group
from azure.cli.core.profiles import ResourceType
from azure.cli.core.azclierror import (
    CLIInternalError,
    InvalidArgumentValueError,
    RequiredArgumentMissingError
)
from azure.core.exceptions import HttpResponseError, ResourceNotFoundError
from azure.mgmt.core.tools import is_valid_resource_id, parse_resource_id
from azure.cli.command_modules.aro._validators import validate_vnet, validate_cidr
from azure.cli.command_modules.aro._rbac import has_role_assignment_on_resource
from azure.cli.command_modules.aro.aaz.latest.network.vnet.subnet import Show as subnet_show
from azure.cli.command_modules.aro.aaz.latest.network.vnet import Show as vnet_show

from knack.log import get_logger


logger = get_logger(__name__)
log_entry_type = {'warn': 'Warning', 'error': 'Error'}


def can_do_action(perms, action):
    for perm in perms:
        matched = False

        for perm_action in perm.actions:
            match = re.escape(perm_action)
            match = re.match("(?i)^" + match.replace(r"\*", ".*") + "$", action)
            if match:
                matched = True
                break

        if not matched:
            continue

        for not_action in perm.not_actions:
            match = re.escape(not_action)
            match = re.match("(?i)^" + match.replace(r"\*", ".*") + "$", action)
            if match:
                matched = False
                break

        if matched:
            return True

    return False


def validate_resource(client, key, resource, actions):
    perms = client.permissions.list_for_resource(resource['resource_group'],
                                                 resource['namespace'],
                                                 "",
                                                 resource['type'],
                                                 resource['name'])
    errors = []
    for action in actions:
        perms, perms_copy = tee(perms)
        perms_list = list(perms_copy)
        if not can_do_action(perms_list, action):
            row = [key, resource['name'], log_entry_type["error"], f"{action} permission is missing"]
            errors.append(row)

    return errors


def get_subnet(cli_ctx, key, subnet, subnet_parts):
    try:
        subnet_obj = subnet_show(cli_ctx=cli_ctx)(command_args={
            'resource_group': subnet_parts['resource_group'],
            'name': subnet_parts['child_name_1'],
            'vnet_name': subnet_parts['name']
        })
    except ResourceNotFoundError as err:
        raise InvalidArgumentValueError(
            f"Invalid --{key.replace('_', '-')}, error when getting '{subnet}': {str(err)}") from err

    except Exception as err:
        raise CLIInternalError(
            f"Unexpected error when getting subnet '{subnet}': {str(err)}") from err

    return subnet_obj


def get_vnet(cli_ctx, key, vnet, vnet_parts):
    try:
        vnet_obj = vnet_show(cli_ctx=cli_ctx)(command_args={
            'name': vnet_parts['name'],
            'resource_group': vnet_parts['resource_group']
        })
    except ResourceNotFoundError as err:
        raise InvalidArgumentValueError(
            f"Invalid --{key.replace('_', '-')}, error when getting '{vnet}': {str(err)}") from err
    except Exception as err:
        raise CLIInternalError(
            f"Unexpected error when getting vnet '{vnet}': {str(err)}") from err

    return vnet_obj


def get_clients(key, cmd):
    parts = parse_resource_id(key)

    auth_client = get_mgmt_service_client(
        cmd.cli_ctx, ResourceType.MGMT_AUTHORIZATION, api_version="2015-07-01")

    return parts, auth_client


# Function to create a progress tracker decorator for the dynamic validation functions
def get_progress_tracker(msg):
    def progress_tracking(func):
        def inner(cmd, namespace):
            hook = cmd.cli_ctx.get_progress_controller()
            hook.add(message=msg)

            errors = func(cmd, namespace)

            hook.end()

            return errors
        return inner
    return progress_tracking


# Validating that the virtual network has the correct permissions
def dyn_validate_vnet(key):
    prog = get_progress_tracker("Validating Virtual Network Permissions")

    @prog
    def _validate_vnet(cmd, namespace):
        errors = []

        vnet = getattr(namespace, key)

        if not is_valid_resource_id(vnet):
            raise RequiredArgumentMissingError(
                f"Must specify --vnet if --{key.replace('_', '-')} is not an id.")

        validate_vnet(cmd, namespace)

        parts, auth_client = get_clients(vnet, cmd)

        get_vnet(cmd.cli_ctx, key, vnet, parts)

        errors = validate_resource(auth_client, key, parts, [
            "Microsoft.Network/virtualNetworks/join/action",
            "Microsoft.Network/virtualNetworks/read",
            "Microsoft.Network/virtualNetworks/write",
            "Microsoft.Network/virtualNetworks/subnets/join/action",
            "Microsoft.Network/virtualNetworks/subnets/read",
            "Microsoft.Network/virtualNetworks/subnets/write", ])

        return errors

    return _validate_vnet


# Validating that the route tables attached to the subnet have the
# correct permissions and that the subnet is not assigned to an NSG
def dyn_validate_subnet_and_route_tables(key):
    prog = get_progress_tracker(f"Validating {key} permissions")

    @prog
    def _validate_subnet(cmd, namespace):
        errors = []

        subnet = getattr(namespace, key)

        if not is_valid_resource_id(subnet):
            if not namespace.vnet:
                raise RequiredArgumentMissingError(
                    f"Must specify --vnet if --{key.replace('_', '-')} is not an id.")

            validate_vnet(cmd, namespace)

            subnet = namespace.vnet + '/subnets/' + subnet
            setattr(namespace, key, subnet)

        parts, auth_client = get_clients(subnet, cmd)

        subnet_obj = get_subnet(cmd.cli_ctx, key, subnet, parts)

        if subnet_obj.get('routeTable', None):
            route_parts = parse_resource_id(subnet_obj['routeTable']['id'])

            errors = validate_resource(auth_client, f"{key}_route_table", route_parts, [
                "Microsoft.Network/routeTables/join/action",
                "Microsoft.Network/routeTables/read",
                "Microsoft.Network/routeTables/write"])

        if not namespace.enable_preconfigured_nsg and subnet_obj.get('networkSecurityGroup', None):
            message = f"A Network Security Group \"{subnet_obj['networkSecurityGroup']['id']}\" "\
                      "is already assigned to this subnet. Ensure there are no Network "\
                      "Security Groups assigned to cluster subnets before cluster creation"
            error = [key, parts['child_name_1'], log_entry_type["error"], message]
            errors.append(error)

        return errors

    return _validate_subnet


# Validating that the cidr ranges between the master_subnet, worker_subnet,
# service_cidr and pod_cidr do not overlap at all
def dyn_validate_cidr_ranges():
    prog = get_progress_tracker("Validating no overlapping CIDR Ranges on subnets")

    @prog
    def _validate_cidr_ranges(cmd, namespace):
        MIN_CIDR_PREFIX = 23

        addresses = []

        ERROR_KEY = "CIDR Range"
        master_subnet = namespace.master_subnet
        worker_subnet = namespace.worker_subnet
        pod_cidr = namespace.pod_cidr
        service_cidr = namespace.service_cidr

        worker_parts = parse_resource_id(worker_subnet)
        master_parts = parse_resource_id(master_subnet)

        fn = validate_cidr("pod_cidr")
        fn(namespace)
        fn = validate_cidr("service_cidr")
        fn(namespace)

        cidr_array = {}

        if pod_cidr is not None:
            node_mask = MIN_CIDR_PREFIX - int(pod_cidr.split("/")[1])
            if node_mask < 2:
                addresses.append(["Pod CIDR",
                                  "Pod CIDR Capacity",
                                  log_entry_type["error"],
                                  f"{pod_cidr} does not contain enough addresses for 3 master nodes " +
                                  "(Requires cidr prefix of 21 or lower)"])
            cidr_array["Pod CIDR"] = ipaddress.IPv4Network(pod_cidr)
        if service_cidr is not None:
            cidr_array["Service CIDR"] = ipaddress.IPv4Network(service_cidr)

        worker_subnet_obj = get_subnet(
            cmd.cli_ctx, None, worker_subnet, worker_parts)

        if worker_subnet_obj.get('addressPrefix', None) is None:
            for address in worker_subnet_obj.get('addressPrefixes'):
                cidr_array["Worker Subnet CIDR -- " +
                           address] = ipaddress.IPv4Network(address)
        else:
            cidr_array["Worker Subnet CIDR"] = ipaddress.IPv4Network(
                worker_subnet_obj.get('addressPrefix'))

        master_subnet_obj = get_subnet(
            cmd.cli_ctx, None, master_subnet, master_parts)

        if master_subnet_obj.get('addressPrefix', None) is None:
            for address in master_subnet_obj.get('addressPrefixes'):
                cidr_array["Master Subnet CIDR -- " +
                           address] = ipaddress.IPv4Network(address)
        else:
            cidr_array["Master Subnet CIDR"] = ipaddress.IPv4Network(
                master_subnet_obj.get('addressPrefix'))

        ipv4_zero = ipaddress.IPv4Network("0.0.0.0/0")

        for item in cidr_array.items():
            key = item[0]
            cidr = item[1]
            if not cidr.overlaps(ipv4_zero):
                addresses.append([ERROR_KEY, key, log_entry_type["error"],
                                  f"{cidr} is not valid as it does not overlap with {ipv4_zero}"])
            for item2 in cidr_array.items():
                compare = item2[1]
                if cidr is not compare:
                    if cidr.overlaps(compare):
                        addresses.append([ERROR_KEY, key, log_entry_type["error"],
                                          f"{cidr} is not valid as it overlaps with {compare}"])

        return addresses

    return _validate_cidr_ranges


def dyn_validate_resource_permissions(service_principal_ids, resources):
    prog = get_progress_tracker("Validating resource permissions")

    @prog
    def _validate_resource_permissions(cmd,
                                       _namespace):
        errors = []

        for sp_id in service_principal_ids:
            for role in sorted(resources):
                for resource in resources[role]:
                    try:
                        resource_contributor_exists = has_role_assignment_on_resource(cmd.cli_ctx,
                                                                                      resource,
                                                                                      sp_id,
                                                                                      role)
                        if not resource_contributor_exists:
                            parts = parse_resource_id(resource)
                            errors.append(["Resource Permissions",
                                           parts['type'],
                                           log_entry_type["warn"],
                                           f"Resource {parts['name']} is missing role assignment " +
                                           f"{role} for service principal {sp_id} " +
                                           "(These roles will be automatically added during cluster creation)"])
                    except HttpResponseError as e:
                        logger.error(e.message)
                        raise
        return errors
    return _validate_resource_permissions


def dyn_validate_version():
    prog = get_progress_tracker("Validating OpenShift Version")

    @prog
    def _validate_version(cmd,
                          namespace):
        errors = []

        if namespace.location is None:
            get_default_location_from_resource_group(cmd, namespace)

        from azure.cli.command_modules.aro.custom import aro_get_versions
        versions = aro_get_versions(namespace.client, namespace.location)

        found = False
        for version in versions:
            if version == namespace.version:
                found = True
                break

        if not found:
            errors.append(["OpenShift Version",
                           namespace.version,
                           log_entry_type["error"],
                           f"{namespace.version} is not a valid version, valid versions are {versions}"])

        return errors
    return _validate_version


def validate_cluster_create(version,
                            resources,
                            service_principal_ids):
    error_object = []

    error_object.append(dyn_validate_vnet("vnet"))
    error_object.append(dyn_validate_subnet_and_route_tables("master_subnet"))
    error_object.append(dyn_validate_subnet_and_route_tables("worker_subnet"))
    error_object.append(dyn_validate_cidr_ranges())
    error_object.append(dyn_validate_resource_permissions(service_principal_ids, resources))
    if version is not None:
        error_object.append(dyn_validate_version())

    return error_object


def dyn_validate_managed_identity_delete_permissions():
    prog = get_progress_tracker("Validating Managed Identity Delete Permissions")

    @prog
    def _validate_managed_identity_delete_permissions(cmd, namespace):
        errors = []
        managed_identities = namespace.managed_identities

        for mi in managed_identities:
            parts, auth_client = get_clients(mi, cmd)
            validation_errors = validate_resource(auth_client, "Managed Identity", parts, [
                "Microsoft.ManagedIdentity/userAssignedIdentities/delete"
            ])
            for error in validation_errors:
                errors.append(f"{error[3]} over {mi}")

        return errors

    return _validate_managed_identity_delete_permissions


def validate_cluster_delete(cmd, delete_identities, managed_identities):
    errors = []

    if delete_identities:
        namespace = collections.namedtuple("Namespace", ["managed_identities"])(managed_identities)
        validate_managed_identity_delete = dyn_validate_managed_identity_delete_permissions()
        errors.extend(validate_managed_identity_delete(cmd, namespace))

    return errors



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import os

from azure.cli.testsdk.scenario_tests import AllowLargeResponse
from azure.cli.testsdk.scenario_tests.const import ENV_LIVE_TEST
from azure.cli.testsdk import (
    JMESPathCheck,
    JMESPathCheckExists,
    ResourceGroupPreparer,
    KeyVaultPreparer,
    ScenarioTest)
from .constants import DEFAULT_LOCATION, SERVER_NAME_PREFIX, SERVER_NAME_MAX_LENGTH


class FlexibleServerIdentityCMKMgmtScenarioTest(ScenarioTest):

    postgres_location = DEFAULT_LOCATION

    @AllowLargeResponse()
    @ResourceGroupPreparer(location=postgres_location)
    @KeyVaultPreparer(name_prefix='pgvault', parameter_name='vault_name', location=postgres_location, additional_params='--enable-purge-protection true --retention-days 90 --no-self-perms')
    def test_postgres_flexible_server_cmk_mgmt(self, resource_group, vault_name):
        self._test_flexible_server_cmk_mgmt(resource_group, vault_name)

    def _test_flexible_server_cmk_mgmt(self, resource_group, vault_name):
        live_test = os.environ.get(ENV_LIVE_TEST, False)
        key_name = self.create_random_name('pgkey', 32)
        identity_name = self.create_random_name('identity', 32)
        server_name = self.create_random_name(SERVER_NAME_PREFIX, SERVER_NAME_MAX_LENGTH)
        tier = 'GeneralPurpose'
        sku_name = 'Standard_D2ds_v4'
        location = self.postgres_location
        replication_role = 'AsyncReplica'
        scope = '/subscriptions/{}/resourceGroups/{}'.format(self.get_subscription_id(), resource_group)

        # Create identity and assign role
        key = self.cmd('keyvault key create --name {} -p software --vault-name {}'
                       .format(key_name, vault_name)).get_output_in_json()

        identity = self.cmd('identity create -g {} --name {} --location {}'.format(resource_group, identity_name, location)).get_output_in_json()
        if (live_test):
            self.cmd('role assignment create --assignee-object-id {} --assignee-principal-type ServicePrincipal --role "Key Vault Crypto User" --scope {}'.format( identity['principalId'], scope))
            self.cmd('role assignment create --assignee-object-id {} --assignee-principal-type ServicePrincipal --role "Key Vault Certificate User" --scope {}'.format( identity['principalId'], scope))

        # Create primary flexible server with data encryption
        self.cmd('postgres flexible-server create -g {} -n {} --public-access none --tier {} --sku-name {} --key {} --identity {} --location {}'.format(
                    resource_group,
                    server_name,
                    tier,
                    sku_name,
                    key['key']['kid'],
                    identity['id'],
                    location
                ))
        
        main_checks = [
            JMESPathCheckExists('identity.userAssignedIdentities."{}"'.format(identity['id']))
        ]

        # Check data encryption is set correctly
        self.cmd('postgres flexible-server show -g {} -n {}'.format(resource_group, server_name),
                 checks=main_checks)

        # Should fail because we can't remove identity used for data encryption
        self.cmd('postgres flexible-server identity remove -g {} -s {} -n {} --yes'
                    .format(resource_group, server_name, identity['id']),
                    expect_failure=True)

        # Create replica 1 with data encryption            
        replica_1_name = self.create_random_name(SERVER_NAME_PREFIX, SERVER_NAME_MAX_LENGTH)

        self.cmd('postgres flexible-server replica create -g {} --name {} --source-server {} --key {} --identity {}'.format(
                    resource_group,
                    replica_1_name,
                    server_name,
                    key['key']['kid'],
                    identity['id']
        ), checks=main_checks + [JMESPathCheck('replicationRole', replication_role)])

        # Clean up
        self.cmd('postgres flexible-server delete -g {} -n {} --yes'.format(resource_group, replica_1_name))
        self.cmd('postgres flexible-server delete -g {} -n {} --yes'.format(resource_group, server_name))


# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import time
from azure.cli.testsdk import ScenarioTest,ResourceGroupPreparer
from azure.cli.testsdk.scenario_tests import record_only


class TestDevice(ScenarioTest):

    @ResourceGroupPreparer(key='rg')
    def test_device(self):
        tag_key = self.create_random_name(prefix='key-', length=10)
        tag_value = self.create_random_name(prefix='val-', length=10)

        self.kwargs.update({
            'name': self.create_random_name(prefix='cli-', length=20),
            'sku': 'Gateway',
            'location': 'eastus',
            'tags': tag_key + '=' + tag_value,
        })

        device = self.cmd('databoxedge device create -l {location} --sku {sku} --name {name} -g {rg}', checks=[
            self.check('name', '{name}'),
            self.check('resourceGroup', '{rg}'),
            self.check('deviceType', 'DataBoxEdgeDevice'),
            self.check('sku.name', '{sku}'),
            self.check('sku.tier', 'Standard'),
            self.check('location', '{location}')
        ]).get_output_in_json()

        self.cmd('databoxedge device update --name {name} -g {rg} --tags {tags}', checks=[
            self.check('id', device['id']),
            self.check('name', '{name}'),
            self.check('resourceGroup', '{rg}'),
            self.check('tags', {tag_key: tag_value})
        ])

        self.cmd('databoxedge device list -g {rg}', checks=[
            self.check('length(@)', 1),
            self.check('type(@)', 'array'),
            self.check('[0].id', device['id']),
            self.check('[0].name', '{name}'),
            self.check('[0].resourceGroup', '{rg}'),
            self.check('[0].tags', {tag_key: tag_value})
        ])

        self.cmd('databoxedge device show -n {name} -g {rg}', checks=[
            self.check('name', '{name}'),
            self.check('resourceGroup', '{rg}'),
            self.check('deviceType', 'DataBoxEdgeDevice'),
            self.check('sku.name', '{sku}'),
            self.check('sku.tier', 'Standard'),
            self.check('location', '{location}'),
            self.check('tags', {tag_key: tag_value})
        ])

        self.cmd('databoxedge device show-update-summary -n {name} -g {rg}', checks=[
            self.check('name', 'default'),
            self.check('resourceGroup', '{rg}')
        ])
        self.cmd('databoxedge alert list -d {name} -g {rg}', checks=self.is_empty())
        self.cmd('databoxedge list-node -g {rg} -d {name}', checks=self.is_empty())
        self.cmd('databoxedge device delete -n {name} -g {rg} -y')
        time.sleep(30)
        self.cmd('databoxedge device list -g {rg}', checks=[self.is_empty()])

    @record_only()
    def test_device_subresource(self):
        self.kwargs.update({
            'share_name': self.create_random_name('share', 10),
            'container_name': self.create_random_name('container', 15)
        })
        self.cmd('az databoxedge device share create -g databoxedge_test -n {share_name} --device-name testdevice1 --access-protocol NFS --monitoring-status Enabled --share-status OK --azure-container-info {{containerName:{container_name},dataFormat:BlockBlob,storageAccountCredentialId:/subscriptions/0b1f6471-1bf0-4dda-aec3-cb9272f09590/resourceGroups/databoxedge_test/providers/Microsoft.DataBoxEdge/dataBoxEdgeDevices/testedgedevice/storageAccountCredentials/asekvlogsase03cf9b66edfc}}', checks=[
            self.check('name', '{share_name}'),
            self.check('accessProtocol', 'NFS'),
            self.check('azureContainerInfo.containerName', '{container_name}'),
            self.check('shareStatus', 'OK')
        ])
        self.cmd('az databoxedge device share update -g databoxedge_test -n {share_name} --device-name testdevice1 --description test', checks=[
            self.check('name', '{share_name}'),
            self.check('accessProtocol', 'NFS'),
            self.check('azureContainerInfo.containerName', '{container_name}'),
            self.check('shareStatus', 'OK'),
            self.check('description', 'test')
        ])
        self.cmd('az databoxedge device share show -g databoxedge_test -n {share_name} --device-name testdevice1', checks=[
            self.check('name', '{share_name}'),
            self.check('accessProtocol', 'NFS'),
            self.check('azureContainerInfo.containerName', '{container_name}'),
            self.check('shareStatus', 'OK'),
            self.check('description', 'test')
        ])
        self.cmd('az databoxedge device share list -g databoxedge_test --device-name testdevice1', checks=[
            self.check('[0].type', 'Microsoft.DataBoxEdge/dataBoxEdgeDevices/shares'),
            self.check('[0].shareStatus', 'OK'),
        ])

        self.cmd('az databoxedge device user list -g databoxedge_test --device-name testdevice1', checks=[
            self.check('[0].type', 'Microsoft.DataBoxEdge/dataBoxEdgeDevices/users'),
            self.check('[0].userType', 'Share')
        ])
        self.cmd('az databoxedge device user show -g databoxedge_test -n testshare1 --device-name testdevice1', checks=[
            self.check('name', 'testshare1'),
            self.check('type', 'Microsoft.DataBoxEdge/dataBoxEdgeDevices/users'),
            self.check('userType', 'Share')
        ])

        self.cmd('az databoxedge device storage-account-credential list -g databoxedge_test --device-name testdevice1', checks=[
            self.check('[0].type', 'Microsoft.DataBoxEdge/dataBoxEdgeDevices/storageAccountCredentials')
        ])
        self.cmd('az databoxedge device storage-account-credential show -g databoxedge_test -n sac1 --device-name testdevice1', checks=[
            self.check('name', 'sac1'),
            self.check('type', 'Microsoft.DataBoxEdge/dataBoxEdgeDevices/storageAccountCredentials'),
            self.check('accountType', 'BlobStorage')
        ])

        self.cmd('az databoxedge device storage-account list -g databoxedge_test --device-name testdevice1')



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

# pylint: skip-file
# flake8: noqa

from azure.cli.core.aaz import *


@register_command(
    "network network-watcher connection-monitor list",
)
class List(AAZCommand):
    """List all connection monitors for the specified Network Watcher.
    """

    _aaz_info = {
        "version": "2023-09-01",
        "resources": [
            ["mgmt-plane", "/subscriptions/{}/resourcegroups/{}/providers/microsoft.network/networkwatchers/{}/connectionmonitors", "2023-09-01"],
        ]
    }

    def _handler(self, command_args):
        super()._handler(command_args)
        self._execute_operations()
        return self._output()

    _args_schema = None

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        if cls._args_schema is not None:
            return cls._args_schema
        cls._args_schema = super()._build_arguments_schema(*args, **kwargs)

        # define Arg Group ""

        _args_schema = cls._args_schema
        _args_schema.network_watcher_name = AAZStrArg(
            options=["--network-watcher-name"],
            help="The name of the Network Watcher resource.",
            required=True,
        )
        _args_schema.resource_group = AAZResourceGroupNameArg(
            required=True,
        )
        return cls._args_schema

    def _execute_operations(self):
        self.pre_operations()
        self.ConnectionMonitorsList(ctx=self.ctx)()
        self.post_operations()

    @register_callback
    def pre_operations(self):
        pass

    @register_callback
    def post_operations(self):
        pass

    def _output(self, *args, **kwargs):
        result = self.deserialize_output(self.ctx.vars.instance.value, client_flatten=True)
        return result

    class ConnectionMonitorsList(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [200]:
                return self.on_200(session)

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Network/networkWatchers/{networkWatcherName}/connectionMonitors",
                **self.url_parameters
            )

        @property
        def method(self):
            return "GET"

        @property
        def error_format(self):
            return "ODataV4Format"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "networkWatcherName", self.ctx.args.network_watcher_name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "resourceGroupName", self.ctx.args.resource_group,
                    required=True,
                ),
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "api-version", "2023-09-01",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        def on_200(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200
            )

        _schema_on_200 = None

        @classmethod
        def _build_schema_on_200(cls):
            if cls._schema_on_200 is not None:
                return cls._schema_on_200

            cls._schema_on_200 = AAZObjectType()

            _schema_on_200 = cls._schema_on_200
            _schema_on_200.value = AAZListType()

            value = cls._schema_on_200.value
            value.Element = AAZObjectType()

            _element = cls._schema_on_200.value.Element
            _element.etag = AAZStrType(
                flags={"read_only": True},
            )
            _element.id = AAZStrType(
                flags={"read_only": True},
            )
            _element.location = AAZStrType()
            _element.name = AAZStrType(
                flags={"read_only": True},
            )
            _element.properties = AAZObjectType(
                flags={"client_flatten": True},
            )
            _element.tags = AAZDictType()
            _element.type = AAZStrType(
                flags={"read_only": True},
            )

            properties = cls._schema_on_200.value.Element.properties
            properties.auto_start = AAZBoolType(
                serialized_name="autoStart",
            )
            properties.connection_monitor_type = AAZStrType(
                serialized_name="connectionMonitorType",
                flags={"read_only": True},
            )
            properties.destination = AAZObjectType()
            properties.endpoints = AAZListType()
            properties.monitoring_interval_in_seconds = AAZIntType(
                serialized_name="monitoringIntervalInSeconds",
            )
            properties.monitoring_status = AAZStrType(
                serialized_name="monitoringStatus",
                flags={"read_only": True},
            )
            properties.notes = AAZStrType()
            properties.outputs = AAZListType()
            properties.provisioning_state = AAZStrType(
                serialized_name="provisioningState",
                flags={"read_only": True},
            )
            properties.source = AAZObjectType()
            properties.start_time = AAZStrType(
                serialized_name="startTime",
                flags={"read_only": True},
            )
            properties.test_configurations = AAZListType(
                serialized_name="testConfigurations",
            )
            properties.test_groups = AAZListType(
                serialized_name="testGroups",
            )

            destination = cls._schema_on_200.value.Element.properties.destination
            destination.address = AAZStrType()
            destination.port = AAZIntType()
            destination.resource_id = AAZStrType(
                serialized_name="resourceId",
            )

            endpoints = cls._schema_on_200.value.Element.properties.endpoints
            endpoints.Element = AAZObjectType()

            _element = cls._schema_on_200.value.Element.properties.endpoints.Element
            _element.address = AAZStrType()
            _element.coverage_level = AAZStrType(
                serialized_name="coverageLevel",
            )
            _element.filter = AAZObjectType()
            _element.name = AAZStrType(
                flags={"required": True},
            )
            _element.resource_id = AAZStrType(
                serialized_name="resourceId",
            )
            _element.scope = AAZObjectType()
            _element.type = AAZStrType()

            filter = cls._schema_on_200.value.Element.properties.endpoints.Element.filter
            filter.items = AAZListType()
            filter.type = AAZStrType()

            items = cls._schema_on_200.value.Element.properties.endpoints.Element.filter.items
            items.Element = AAZObjectType()

            _element = cls._schema_on_200.value.Element.properties.endpoints.Element.filter.items.Element
            _element.address = AAZStrType()
            _element.type = AAZStrType()

            scope = cls._schema_on_200.value.Element.properties.endpoints.Element.scope
            scope.exclude = AAZListType()
            scope.include = AAZListType()

            exclude = cls._schema_on_200.value.Element.properties.endpoints.Element.scope.exclude
            exclude.Element = AAZObjectType()
            _ListHelper._build_schema_connection_monitor_endpoint_scope_item_read(exclude.Element)

            include = cls._schema_on_200.value.Element.properties.endpoints.Element.scope.include
            include.Element = AAZObjectType()
            _ListHelper._build_schema_connection_monitor_endpoint_scope_item_read(include.Element)

            outputs = cls._schema_on_200.value.Element.properties.outputs
            outputs.Element = AAZObjectType()

            _element = cls._schema_on_200.value.Element.properties.outputs.Element
            _element.type = AAZStrType()
            _element.workspace_settings = AAZObjectType(
                serialized_name="workspaceSettings",
            )

            workspace_settings = cls._schema_on_200.value.Element.properties.outputs.Element.workspace_settings
            workspace_settings.workspace_resource_id = AAZStrType(
                serialized_name="workspaceResourceId",
            )

            source = cls._schema_on_200.value.Element.properties.source
            source.port = AAZIntType()
            source.resource_id = AAZStrType(
                serialized_name="resourceId",
                flags={"required": True},
            )

            test_configurations = cls._schema_on_200.value.Element.properties.test_configurations
            test_configurations.Element = AAZObjectType()

            _element = cls._schema_on_200.value.Element.properties.test_configurations.Element
            _element.http_configuration = AAZObjectType(
                serialized_name="httpConfiguration",
            )
            _element.icmp_configuration = AAZObjectType(
                serialized_name="icmpConfiguration",
            )
            _element.name = AAZStrType(
                flags={"required": True},
            )
            _element.preferred_ip_version = AAZStrType(
                serialized_name="preferredIPVersion",
            )
            _element.protocol = AAZStrType(
                flags={"required": True},
            )
            _element.success_threshold = AAZObjectType(
                serialized_name="successThreshold",
            )
            _element.tcp_configuration = AAZObjectType(
                serialized_name="tcpConfiguration",
            )
            _element.test_frequency_sec = AAZIntType(
                serialized_name="testFrequencySec",
            )

            http_configuration = cls._schema_on_200.value.Element.properties.test_configurations.Element.http_configuration
            http_configuration.method = AAZStrType()
            http_configuration.path = AAZStrType()
            http_configuration.port = AAZIntType()
            http_configuration.prefer_https = AAZBoolType(
                serialized_name="preferHTTPS",
            )
            http_configuration.request_headers = AAZListType(
                serialized_name="requestHeaders",
            )
            http_configuration.valid_status_code_ranges = AAZListType(
                serialized_name="validStatusCodeRanges",
            )

            request_headers = cls._schema_on_200.value.Element.properties.test_configurations.Element.http_configuration.request_headers
            request_headers.Element = AAZObjectType()

            _element = cls._schema_on_200.value.Element.properties.test_configurations.Element.http_configuration.request_headers.Element
            _element.name = AAZStrType()
            _element.value = AAZStrType()

            valid_status_code_ranges = cls._schema_on_200.value.Element.properties.test_configurations.Element.http_configuration.valid_status_code_ranges
            valid_status_code_ranges.Element = AAZStrType()

            icmp_configuration = cls._schema_on_200.value.Element.properties.test_configurations.Element.icmp_configuration
            icmp_configuration.disable_trace_route = AAZBoolType(
                serialized_name="disableTraceRoute",
            )

            success_threshold = cls._schema_on_200.value.Element.properties.test_configurations.Element.success_threshold
            success_threshold.checks_failed_percent = AAZIntType(
                serialized_name="checksFailedPercent",
            )
            success_threshold.round_trip_time_ms = AAZFloatType(
                serialized_name="roundTripTimeMs",
            )

            tcp_configuration = cls._schema_on_200.value.Element.properties.test_configurations.Element.tcp_configuration
            tcp_configuration.destination_port_behavior = AAZStrType(
                serialized_name="destinationPortBehavior",
            )
            tcp_configuration.disable_trace_route = AAZBoolType(
                serialized_name="disableTraceRoute",
            )
            tcp_configuration.port = AAZIntType()

            test_groups = cls._schema_on_200.value.Element.properties.test_groups
            test_groups.Element = AAZObjectType()

            _element = cls._schema_on_200.value.Element.properties.test_groups.Element
            _element.destinations = AAZListType(
                flags={"required": True},
            )
            _element.disable = AAZBoolType()
            _element.name = AAZStrType(
                flags={"required": True},
            )
            _element.sources = AAZListType(
                flags={"required": True},
            )
            _element.test_configurations = AAZListType(
                serialized_name="testConfigurations",
                flags={"required": True},
            )

            destinations = cls._schema_on_200.value.Element.properties.test_groups.Element.destinations
            destinations.Element = AAZStrType()

            sources = cls._schema_on_200.value.Element.properties.test_groups.Element.sources
            sources.Element = AAZStrType()

            test_configurations = cls._schema_on_200.value.Element.properties.test_groups.Element.test_configurations
            test_configurations.Element = AAZStrType()

            tags = cls._schema_on_200.value.Element.tags
            tags.Element = AAZStrType()

            return cls._schema_on_200


class _ListHelper:
    """Helper class for List"""

    _schema_connection_monitor_endpoint_scope_item_read = None

    @classmethod
    def _build_schema_connection_monitor_endpoint_scope_item_read(cls, _schema):
        if cls._schema_connection_monitor_endpoint_scope_item_read is not None:
            _schema.address = cls._schema_connection_monitor_endpoint_scope_item_read.address
            return

        cls._schema_connection_monitor_endpoint_scope_item_read = _schema_connection_monitor_endpoint_scope_item_read = AAZObjectType()

        connection_monitor_endpoint_scope_item_read = _schema_connection_monitor_endpoint_scope_item_read
        connection_monitor_endpoint_scope_item_read.address = AAZStrType()

        _schema.address = cls._schema_connection_monitor_endpoint_scope_item_read.address


__all__ = ["List"]



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.testsdk import (LiveScenarioTest, ResourceGroupPreparer, StorageAccountPreparer)
from azure.cli.testsdk.scenario_tests.decorators import AllowLargeResponse


class NWFlowLogScenarioTest(LiveScenarioTest):

    @ResourceGroupPreparer(name_prefix='test_nw_flow_log_', location='centraluseuap')
    @StorageAccountPreparer(name_prefix='testflowlog', location='centraluseuap', kind='StorageV2')
    def test_nw_flow_log_create_vnetfl(self, resource_group, resource_group_location, storage_account):
        self.kwargs.update({
            'rg': resource_group,
            'location': resource_group_location,
            'storage_account': storage_account,
            'vnet': 'vnet1',
            'subnet': 'subnet1',
            'nic': 'nic1',
            'watcher_rg': 'NetworkWatcherRG',
            'watcher_name': 'NetworkWatcher_{}'.format(resource_group_location),
            'flow_log': 'flow_log_test',
            'workspace': self.create_random_name('clitest', 20),
        })

        # enable network watcher
        # self.cmd('network watcher configure -g {rg} --locations {location} --enabled')

        # prepare the target resource
        self.cmd('network vnet create -g {rg} -n {vnet}')
        self.cmd('network vnet subnet create -g {rg} --vnet-name {vnet} -n {subnet} --address-prefix 10.0.0.0/24')
        self.cmd('network nic create -g {rg} -n {nic} --vnet-name {vnet} --subnet {subnet}')

        # prepare workspace
        workspace = self.cmd('monitor log-analytics workspace create '
                             '--resource-group {rg} '
                             '--location eastus '
                             '--workspace-name {workspace} ').get_output_in_json()
        self.kwargs.update({
            'workspace_id': workspace['id']
        })

        #targetId as vnet
        self.cmd('network watcher flow-log create '
                 '--location {location} '
                 '--resource-group {rg} '
                 '--vnet {vnet} '
                 '--storage-account {storage_account} '
                 '--workspace {workspace_id} '
                 '--name {flow_log} ')

        self.cmd('network watcher flow-log list --location {location}')

        # This output is Azure Management Resource formatted.
        self.cmd('network watcher flow-log show --location {location} --name {flow_log}', checks=[
            self.check('name', self.kwargs['flow_log']),
            self.check('flowAnalyticsConfiguration.networkWatcherFlowAnalyticsConfiguration.workspaceResourceId',
                       self.kwargs['workspace_id']),
            self.check_pattern('targetResourceId', '.*/{vnet}$'),
            self.check('retentionPolicy.days', 0),
            self.check('retentionPolicy.enabled', False),
        ])

        #targetId as subnet
        self.cmd('network watcher flow-log create '
                 '--location {location} '
                 '--resource-group {rg} '
                 '--vnet {vnet} '
                 '--subnet {subnet} '
                 '--storage-account {storage_account} '
                 '--workspace {workspace_id} '
                 '--name {flow_log} ')

        self.cmd('network watcher flow-log list --location {location}')

        # This output is Azure Management Resource formatted.
        self.cmd('network watcher flow-log show --location {location} --name {flow_log}', checks=[
            self.check('name', self.kwargs['flow_log']),
            self.check('flowAnalyticsConfiguration.networkWatcherFlowAnalyticsConfiguration.workspaceResourceId',
                       self.kwargs['workspace_id']),
            self.check_pattern('targetResourceId', '.*/{subnet}$'),
            self.check('retentionPolicy.days', 0),
            self.check('retentionPolicy.enabled', False),
        ])
        
        #targetId as nic
        self.cmd('network watcher flow-log create '
                 '--location {location} '
                 '--resource-group {rg} '
                 '--nic {nic} '
                 '--storage-account {storage_account} '
                 '--workspace {workspace_id} '
                 '--name {flow_log} ')

        self.cmd('network watcher flow-log list --location {location}')

        # This output is Azure Management Resource formatted.
        self.cmd('network watcher flow-log show --location {location} --name {flow_log}', checks=[
            self.check('name', self.kwargs['flow_log']),
            self.check('flowAnalyticsConfiguration.networkWatcherFlowAnalyticsConfiguration.workspaceResourceId',
                       self.kwargs['workspace_id']),
            self.check_pattern('targetResourceId', '.*/{nic}$'),
            self.check('retentionPolicy.days', 0),
            self.check('retentionPolicy.enabled', False),
        ])

    @ResourceGroupPreparer(name_prefix='test_nw_flow_log_', location='centraluseuap')
    @StorageAccountPreparer(name_prefix='testflowlog', location='centraluseuap', kind='StorageV2')
    def test_nw_flow_log_create_vnetfl_with_filtering(self, resource_group, resource_group_location, storage_account):
        self.kwargs.update({
            'rg': resource_group,
            'location': resource_group_location,
            'storage_account': storage_account,
            'vnet': 'vnet1',
            'subnet': 'subnet1',
            'nic': 'nic1',
            'watcher_rg': 'NetworkWatcherRG',
            'watcher_name': 'NetworkWatcher_{}'.format(resource_group_location),
            'flow_log': 'flow_log_test',
            'filtering_criteria': 'dstip=20.252.145.59 || DstPort=8080',
            'workspace': self.create_random_name('clitest', 20),
        })

        # enable network watcher
        # self.cmd('network watcher configure -g {rg} --locations {location} --enabled')

        # prepare the target resource
        self.cmd('network vnet create -g {rg} -n {vnet}')
        self.cmd('network vnet subnet create -g {rg} --vnet-name {vnet} -n {subnet} --address-prefix 10.0.0.0/24')
        self.cmd('network nic create -g {rg} -n {nic} --vnet-name {vnet} --subnet {subnet}')

        # prepare workspace
        workspace = self.cmd('monitor log-analytics workspace create '
                             '--resource-group {rg} '
                             '--location eastus '
                             '--workspace-name {workspace} ').get_output_in_json()
        self.kwargs.update({
            'workspace_id': workspace['id']
        })

        #targetId as vnet
        self.cmd('network watcher flow-log create '
                 '--location {location} '
                 '--resource-group {rg} '
                 '--vnet {vnet} '
                 '--storage-account {storage_account} '
                 '--filtering-criteria {filtering_criteria} '
                 '--workspace {workspace_id} '
                 '--name {flow_log} ')

        self.cmd('network watcher flow-log list --location {location}')

        # This output is Azure Management Resource formatted.
        self.cmd('network watcher flow-log show --location {location} --name {flow_log}', checks=[
            self.check('name', self.kwargs['flow_log']),
            self.check('enabledFilteringCriteria', '.*/{filtering_criteria}$'),
            self.check('flowAnalyticsConfiguration.networkWatcherFlowAnalyticsConfiguration.workspaceResourceId',
                       self.kwargs['workspace_id']),
            self.check_pattern('targetResourceId', '.*/{vnet}$'),
            self.check('retentionPolicy.days', 0),
            self.check('retentionPolicy.enabled', False),
        ])

    @ResourceGroupPreparer(name_prefix='test_nw_flow_log_', location='centraluseuap')
    @StorageAccountPreparer(name_prefix='testflowlog', location='centraluseuap', kind='StorageV2')
    def test_nw_flow_log_create_vnetfl_with_record_types(self, resource_group, resource_group_location, storage_account):
        self.kwargs.update({
            'rg': resource_group,
            'location': resource_group_location,
            'storage_account': storage_account,
            'vnet': 'vnet1',
            'subnet': 'subnet1',
            'nic': 'nic1',
            'watcher_rg': 'NetworkWatcherRG',
            'watcher_name': 'NetworkWatcher_{}'.format(resource_group_location),
            'flow_log': 'flow_log_test',
            'record_types': 'B,E',
            'workspace': self.create_random_name('clitest', 20),
        })

        # enable network watcher
        # self.cmd('network watcher configure -g {rg} --locations {location} --enabled')

        # prepare the target resource
        self.cmd('network vnet create -g {rg} -n {vnet}')
        self.cmd('network vnet subnet create -g {rg} --vnet-name {vnet} -n {subnet} --address-prefix 10.0.0.0/24')
        self.cmd('network nic create -g {rg} -n {nic} --vnet-name {vnet} --subnet {subnet}')

        # prepare workspace
        workspace = self.cmd('monitor log-analytics workspace create '
                             '--resource-group {rg} '
                             '--location eastus '
                             '--workspace-name {workspace} ').get_output_in_json()
        self.kwargs.update({
            'workspace_id': workspace['id']
        })

        # targetId as vnet
        self.cmd('network watcher flow-log create '
                 '--location {location} '
                 '--resource-group {rg} '
                 '--vnet {vnet} '
                 '--storage-account {storage_account} '
                 '--record-types {record_types} '
                 '--workspace {workspace_id} '
                 '--name {flow_log} ')

        self.cmd('network watcher flow-log list --location {location}')

        # This output is Azure Management Resource formatted.
        self.cmd('network watcher flow-log show --location {location} --name {flow_log}', checks=[
            self.check('name', self.kwargs['flow_log']),
            self.check('recordTypes', '{record_types}'),
            self.check('flowAnalyticsConfiguration.networkWatcherFlowAnalyticsConfiguration.workspaceResourceId',
                       self.kwargs['workspace_id']),
            self.check_pattern('targetResourceId', '.*/{vnet}$'),
            self.check('retentionPolicy.days', 0),
            self.check('retentionPolicy.enabled', False),
        ])
    
    @ResourceGroupPreparer(name_prefix='test_nw_flow_log_', location='centraluseuap')
    @StorageAccountPreparer(name_prefix='testflowlog', location='centraluseuap', kind='StorageV2')
    def test_nw_flow_log_update_vnetfl_with_record_types(self, resource_group, resource_group_location, storage_account):
        self.kwargs.update({
            'rg': resource_group,
            'location': resource_group_location,
            'storage_account': storage_account,
            'vnet': 'vnet1',
            'subnet': 'subnet1',
            'nic': 'nic1',
            'watcher_rg': 'NetworkWatcherRG',
            'watcher_name': 'NetworkWatcher_{}'.format(resource_group_location),
            'flow_log': 'flow_log_test_update_rt',
            'workspace': self.create_random_name('clitest', 20),
            'record_types': 'B,E'
        })

        # Prepare target resources
        self.cmd('network vnet create -g {rg} -n {vnet}')
        self.cmd('network vnet subnet create -g {rg} --vnet-name {vnet} -n {subnet} --address-prefix 10.0.0.0/24')
        self.cmd('network nic create -g {rg} -n {nic} --vnet-name {vnet} --subnet {subnet}')

        # Prepare workspace (use eastus for LA workspace)
        workspace = self.cmd(
            'monitor log-analytics workspace create '
            '--resource-group {rg} '
            '--location eastus '
            '--workspace-name {workspace} '
        ).get_output_in_json()
        self.kwargs.update({'workspace_id': workspace['id']})

        # Create flow log for VNet (no record-types initially)
        self.cmd(
            'network watcher flow-log create '
            '--location {location} '
            '--resource-group {rg} '
            '--vnet {vnet} '
            '--storage-account {storage_account} '
            '--workspace {workspace_id} '
            '--name {flow_log} '
        )

        # Baseline verification
        res1 = self.cmd(
            'network watcher flow-log show --location {location} --name {flow_log}'
        ).get_output_in_json()
        self.assertEqual(res1['name'], self.kwargs['flow_log'])
        self.assertEqual(res1['enabled'], True)
        self.assertTrue(res1['targetResourceId'].endswith(self.kwargs['vnet']))

        # ---- Update: set record types to B,E on the same VNet target ----
        res2 = self.cmd(
            'network watcher flow-log update '
            '--location {location} '
            '--resource-group {rg} '
            '--vnet {vnet} '
            '--name {flow_log} '
            '--record-types {record_types} '
        ).get_output_in_json()

        self.assertEqual(res2['name'], self.kwargs['flow_log'])
        self.assertEqual(res2['enabled'], True)
        self.assertTrue(res2['targetResourceId'].endswith(self.kwargs['vnet']))
        self.assertEqual(res2['recordTypes'], self.kwargs['record_types'])
        # Retention defaults remain unless explicitly changed
        self.assertEqual(res2['retentionPolicy']['enabled'], False)
        self.assertEqual(res2['retentionPolicy']['days'], 0)

        # Show again and validate persisted update
        self.cmd('network watcher flow-log show --location {location} --name {flow_log}', checks=[
            self.check('name', self.kwargs['flow_log']),
            self.check('recordTypes', '{record_types}')
        ])

    @ResourceGroupPreparer(name_prefix='test_nw_flow_log_', location='centraluseuap')
    @StorageAccountPreparer(name_prefix='testflowlog', location='centraluseuap', kind='StorageV2')
    def test_nw_flow_log_create_vnetflWithManagedIdentity(self, resource_group, resource_group_location, storage_account):
        self.kwargs.update({
            'rg': resource_group,
            'location': resource_group_location,
            'storage_account': storage_account,
            'vnet': 'vnet1',
            'subnet': 'subnet1',
            'nic': 'nic1',
            'watcher_rg': 'NetworkWatcherRG',
            'watcher_name': 'NetworkWatcher_{}'.format(resource_group_location),
            'flow_log': 'flow_log_test',
            'identity': 'idtest',
            'subscription': 'af15e575-f948-49ac-bce0-252d028e9379'
        })

        # create an managed identity
        self.cmd('identity create -g {rg} -n {identity}').get_output_in_json()

        # enable network watcher
        # self.cmd('network watcher configure -g {rg} --locations {location} --enabled')

        # prepare the target resource
        self.cmd('network vnet create -g {rg} -n {vnet}')

        #targetId as vnet
        self.cmd('network watcher flow-log create '
                 '--location {location} '
                 '--resource-group {rg} '
                 '--vnet {vnet} '
                 '--storage-account {storage_account} '
                 '--name {flow_log} '
                 '--user-assigned-identity /subscriptions/{subscription}/resourceGroups/{rg}/providers/Microsoft.ManagedIdentity/userAssignedIdentities/{identity}')

        self.cmd('network watcher flow-log list --location {location}')

        # This output is Azure Management Resource formatted.
        self.cmd('network watcher flow-log show --location {location} --name {flow_log}', checks=[
            self.check('name', self.kwargs['flow_log']),
           
            self.check_pattern('targetResourceId', '.*/{vnet}$'),
            self.check('retentionPolicy.days', 0),
            self.check('retentionPolicy.enabled', False),
            self.check('identity != null', True),
            self.check('identity.type', 'userAssigned'),
            self.check('identity.userAssignedIdentities != null', True),
            # Check that userAssignedIdentities has at least one entry
            self.check("length(identity.userAssignedIdentities)", 1),
            self.check("identity.userAssignedIdentities.*.clientId | [0] != ''", True),  # Ensure clientId is not null
            self.check("identity.userAssignedIdentities.*.principalId | [0] != ''", True),  # Ensure principalId is not null
        ])

    @ResourceGroupPreparer(name_prefix='test_nw_flow_log_', location='centraluseuap')
    @StorageAccountPreparer(name_prefix='testflowlog', location='centraluseuap', kind='StorageV2')
    def test_nw_flow_log_create_vnetflWithNoneManagedIdentity(self, resource_group, resource_group_location, storage_account):
        self.kwargs.update({
            'rg': resource_group,
            'location': resource_group_location,
            'storage_account': storage_account,
            'vnet': 'vnet1',
            'subnet': 'subnet1',
            'nic': 'nic1',
            'watcher_rg': 'NetworkWatcherRG',
            'watcher_name': 'NetworkWatcher_{}'.format(resource_group_location),
            'flow_log': 'flow_log_test',
            'identity': 'idtest',
            'subscription': 'af15e575-f948-49ac-bce0-252d028e9379',
            'format' : 'JSON'
        })

        # create an managed identity
        self.cmd('identity create -g {rg} -n {identity}').get_output_in_json()

        # enable network watcher
        # self.cmd('network watcher configure -g {rg} --locations {location} --enabled')

        # prepare the target resource
        self.cmd('network vnet create -g {rg} -n {vnet}')

        #targetId as vnet
        self.cmd('network watcher flow-log create '
                 '--location {location} '
                 '--resource-group {rg} '
                 '--vnet {vnet} '
                 '--storage-account {storage_account} '
                 '--name {flow_log} '
                 '--user-assigned-identity /subscriptions/{subscription}/resourceGroups/{rg}/providers/Microsoft.ManagedIdentity/userAssignedIdentities/{identity}')

        self.cmd('network watcher flow-log list --location {location}')

        # This output is Azure Management Resource formatted.
        self.cmd('network watcher flow-log show --location {location} --name {flow_log}', checks=[
            self.check('name', self.kwargs['flow_log']),
           
            self.check_pattern('targetResourceId', '.*/{vnet}$'),
            self.check('retentionPolicy.days', 0),
            self.check('retentionPolicy.enabled', False),
            self.check('identity != null', True),
            self.check('identity.type', 'userAssigned'),
            self.check('identity.userAssignedIdentities != null', True),
            # Check that userAssignedIdentities has at least one entry
            self.check("length(identity.userAssignedIdentities)", 1),
            self.check("identity.userAssignedIdentities.*.clientId | [0] != ''", True),  # Ensure clientId is not null
            self.check("identity.userAssignedIdentities.*.principalId | [0] != ''", True),  # Ensure principalId is not null
        ])

        self.cmd('network watcher flow-log update '
                 '--location {location} '
                 '--resource-group {rg} '
                 '--vnet {vnet} '
                 '--storage-account {storage_account} '
                 '--name {flow_log} '
                 '--format {format} '
                 '--user-assigned-identity none ')
        
        self.cmd('network watcher flow-log list --location {location}')

        # This output is Azure Management Resource formatted.
        self.cmd('network watcher flow-log show --location {location} --name {flow_log}', checks=[
            self.check('name', self.kwargs['flow_log']),
           
            self.check_pattern('targetResourceId', '.*/{vnet}$'),
            self.check('retentionPolicy.days', 0),
            self.check('retentionPolicy.enabled', False),
            self.check('identity != null', True),
            self.check('identity.type', 'none'),
            self.check('identity.userAssignedIdentities', None),
        ])


    @ResourceGroupPreparer(name_prefix='test_nw_flow_log_', location='eastus')
    @StorageAccountPreparer(name_prefix='testflowlog', location='eastus', kind='StorageV2')
    def test_nw_flow_log_create(self, resource_group, resource_group_location, storage_account):
        self.kwargs.update({
            'rg': resource_group,
            'location': resource_group_location,
            'storage_account': storage_account,
            'nsg': 'nsg1',
            'watcher_rg': 'NetworkWatcherRG',
            'watcher_name': 'NetworkWatcher_{}'.format(resource_group_location),
            'flow_log': 'flow_log_test',
            'workspace': self.create_random_name('clitest', 20),
        })

        # enable network watcher
        # self.cmd('network watcher configure -g {rg} --locations {location} --enabled')

        # prepare the target resource
        self.cmd('network nsg create -g {rg} -n {nsg}')

        # prepare workspace
        workspace = self.cmd('monitor log-analytics workspace create '
                             '--resource-group {rg} '
                             '--location {location} '
                             '--workspace-name {workspace} ').get_output_in_json()
        self.kwargs.update({
            'workspace_id': workspace['id']
        })

        self.cmd('network watcher flow-log create '
                 '--location {location} '
                 '--resource-group {rg} '
                 '--nsg {nsg} '
                 '--storage-account {storage_account} '
                 '--workspace {workspace_id} '
                 '--name {flow_log} ')

        self.cmd('network watcher flow-log list --location {location}')

        # This output is Azure Management Resource formatted.
        self.cmd('network watcher flow-log show --location {location} --name {flow_log}', checks=[
            self.check('name', self.kwargs['flow_log']),
            self.check('flowAnalyticsConfiguration.networkWatcherFlowAnalyticsConfiguration.enabled', False),
            self.check('flowAnalyticsConfiguration.networkWatcherFlowAnalyticsConfiguration.workspaceResourceId',
                       self.kwargs['workspace_id']),
            self.check('retentionPolicy.days', 0),
            self.check('retentionPolicy.enabled', False),
        ])
    
    @ResourceGroupPreparer(name_prefix='test_nw_flow_log_', location='centraluseuap')
    @StorageAccountPreparer(name_prefix='testflowlog', location='centraluseuap', kind='StorageV2')
    @AllowLargeResponse(1024)
    def test_nw_flow_log_delete_vnetfl(self, resource_group, resource_group_location, storage_account):
        self.kwargs.update({
            'rg': resource_group,
            'location': resource_group_location,
            'storage_account': storage_account,
            'vnet': 'vnet1',
            'subnet': 'subnet1',
            'nic': 'nic1',
            'watcher_rg': 'NetworkWatcherRG',
            'watcher_name': 'NetworkWatcher_{}'.format(resource_group_location),
            'flow_log': 'flow_log_test2',
            'workspace': self.create_random_name('clitest', 20),
        })

        # enable network watcher
        # self.cmd('network watcher configure -g {rg} --locations {location} --enabled')

        # prepare the target resource
        self.cmd('network vnet create -g {rg} -n {vnet}')
        self.cmd('network vnet subnet create -g {rg} --vnet-name {vnet} -n {subnet} --address-prefix 10.0.0.0/24')
        self.cmd('network nic create -g {rg} -n {nic} --vnet-name {vnet} --subnet {subnet}')

        # prepare workspace
        workspace = self.cmd('monitor log-analytics workspace create '
                             '--resource-group {rg} '
                             '--location westus '
                             '--workspace-name {workspace} ').get_output_in_json()
        self.kwargs.update({
            'workspace_id': workspace['id']
        })

        #targetId as vnet
        self.cmd('network watcher flow-log create '
                 '--location {location} '
                 '--resource-group {rg} '
                 '--vnet {vnet} '
                 '--storage-account {storage_account} '
                 '--workspace {workspace_id} '
                 '--name {flow_log} ')

        self.cmd('network watcher flow-log show --location {location} --name {flow_log}')

        self.cmd('network watcher flow-log delete --location {location} --name {flow_log}')

        with self.assertRaisesRegex(SystemExit, '3'):
            self.cmd('network watcher flow-log show --location {location} --name {flow_log}')
        
        #targetId as subnet
        self.cmd('network watcher flow-log create '
                 '--location {location} '
                 '--resource-group {rg} '
                 '--vnet {vnet} '
                 '--subnet {subnet} '
                 '--storage-account {storage_account} '
                 '--workspace {workspace_id} '
                 '--name {flow_log} ')

        self.cmd('network watcher flow-log show --location {location} --name {flow_log}')

        self.cmd('network watcher flow-log delete --location {location} --name {flow_log}')

        with self.assertRaisesRegex(SystemExit, '3'):
            self.cmd('network watcher flow-log show --location {location} --name {flow_log}')

        
        #targetId as nic
        self.cmd('network watcher flow-log create '
                 '--location {location} '
                 '--resource-group {rg} '
                 '--nic {nic} '
                 '--storage-account {storage_account} '
                 '--workspace {workspace_id} '
                 '--name {flow_log} ')

        self.cmd('network watcher flow-log show --location {location} --name {flow_log}')

        self.cmd('network watcher flow-log delete --location {location} --name {flow_log}')

        with self.assertRaisesRegex(SystemExit, '3'):
            self.cmd('network watcher flow-log show --location {location} --name {flow_log}')
        
    @ResourceGroupPreparer(name_prefix='test_nw_flow_log_', location='eastus')
    @StorageAccountPreparer(name_prefix='testflowlog', location='eastus', kind='StorageV2')
    @AllowLargeResponse(1024)
    def test_nw_flow_log_delete(self, resource_group, resource_group_location, storage_account):
        self.kwargs.update({
            'rg': resource_group,
            'location': resource_group_location,
            'storage_account': storage_account,
            'nsg': 'nsg1',
            'watcher_rg': 'NetworkWatcherRG',
            'watcher_name': 'NetworkWatcher_{}'.format(resource_group_location),
            'flow_log': 'flow_log_test2',
            'workspace': self.create_random_name('clitest', 20),
        })

        # enable network watcher
        # self.cmd('network watcher configure -g {rg} --locations {location} --enabled')

        # prepare the target resource
        self.cmd('network nsg create -g {rg} -n {nsg}')

        # prepare workspace
        workspace = self.cmd('monitor log-analytics workspace create '
                             '--resource-group {rg} '
                             '--location {location} '
                             '--workspace-name {workspace} ').get_output_in_json()
        self.kwargs.update({
            'workspace_id': workspace['id']
        })

        self.cmd('network watcher flow-log create '
                 '--location {location} '
                 '--resource-group {rg} '
                 '--nsg {nsg} '
                 '--storage-account {storage_account} '
                 '--workspace {workspace_id} '
                 '--name {flow_log} ')

        self.cmd('network watcher flow-log show --location {location} --name {flow_log}')

        self.cmd('network watcher flow-log delete --location {location} --name {flow_log}')

        with self.assertRaisesRegex(SystemExit, '3'):
            self.cmd('network watcher flow-log show --location {location} --name {flow_log}')

    @ResourceGroupPreparer(name_prefix='test_nw_flow_log_', location='westus')
    @StorageAccountPreparer(name_prefix='testflowlog', location='westus', kind='StorageV2')
    @AllowLargeResponse(1024)
    def test_nw_flow_log_show(self, resource_group, resource_group_location, storage_account):
        """
        This test is used to demonstrate different outputs between the new and deprecating parameters
        :param resource_group:
        :param resource_group_location:
        :param storage_account:
        :return:
        """
        self.kwargs.update({
            'rg': resource_group,
            'location': resource_group_location,
            'storage_account': storage_account,
            'nsg': 'nsg1',
            'watcher_rg': 'NetworkWatcherRG',
            'watcher_name': 'NetworkWatcher_{}'.format(resource_group_location),
            'flow_log': 'flow_log_test2',
            'workspace': self.create_random_name('clitest', 20),
        })

        # enable network watcher
        # self.cmd('network watcher configure -g {rg} --locations {location} --enabled')

        # prepare the target resource
        nsg_info = self.cmd('network nsg create -g {rg} -n {nsg}').get_output_in_json()
        self.kwargs.update({
            'nsg_id': nsg_info['NewNSG']['id']
        })

        # prepare workspace
        workspace = self.cmd('monitor log-analytics workspace create '
                             '--resource-group {rg} '
                             '--location {location} '
                             '--workspace-name {workspace} ').get_output_in_json()
        self.kwargs.update({
            'workspace_id': workspace['id']
        })

        self.cmd('network watcher flow-log create '
                 '--location {location} '
                 '--resource-group {rg} '
                 '--nsg {nsg} '
                 '--storage-account {storage_account} '
                 '--workspace {workspace_id} '
                 '--name {flow_log} ')

        # This output is new Azure Management Resource formatted.
        self.cmd('network watcher flow-log show --location {location} --name {flow_log}', checks=[
            self.check('name', self.kwargs['flow_log']),
            self.check('enabled', True),
            self.check('format.type', 'JSON'),
            self.check('format.version', 1),
            self.check('flowAnalyticsConfiguration.networkWatcherFlowAnalyticsConfiguration.enabled', False),
            self.check('flowAnalyticsConfiguration.networkWatcherFlowAnalyticsConfiguration.workspaceResourceId',
                       self.kwargs['workspace_id']),
            self.check('retentionPolicy.days', 0),
            self.check('retentionPolicy.enabled', False),
        ])

        # This output is deprecating
        self.cmd('network watcher flow-log show --nsg {nsg_id}', checks=[
            self.check('enabled', True),
            self.check('format.type', 'JSON'),
            self.check('format.version', 1),
            self.check('flowAnalyticsConfiguration.networkWatcherFlowAnalyticsConfiguration.enabled', False),
            self.check('flowAnalyticsConfiguration.networkWatcherFlowAnalyticsConfiguration.workspaceResourceId',
                       self.kwargs['workspace_id']),
            self.check('retentionPolicy.days', 0),
            self.check('retentionPolicy.enabled', False)
        ])

    @ResourceGroupPreparer(name_prefix='test_nw_flow_log_', location='centraluseuap')
    @StorageAccountPreparer(name_prefix='testflowlog', location='centraluseuap', kind='StorageV2')
    def test_nw_flow_log_update_vnetfl(self, resource_group, resource_group_location, storage_account):
        self.kwargs.update({
            'rg': resource_group,
            'location': resource_group_location,
            'storage_account': storage_account,
            'nsg': 'nsg1',
            'vnet': 'vnet1',
            'subnet': 'subnet1',
            'nic': 'nic1',
            'watcher_rg': 'NetworkWatcherRG',
            'watcher_name': 'NetworkWatcher_{}'.format(resource_group_location),
            'flow_log': 'flow_log_test2',
            'workspace': self.create_random_name('clitest', 20),
        })

        # enable network watcher
        # self.cmd('network watcher configure -g {rg} --locations {location} --enabled')

        # prepare the target resource
        self.cmd('network nsg create -g {rg} -n {nsg}')
        self.cmd('network vnet create -g {rg} -n {vnet}')
        self.cmd('network vnet subnet create -g {rg} --vnet-name {vnet} -n {subnet} --address-prefix 10.0.0.0/24')
        self.cmd('network nic create -g {rg} -n {nic} --vnet-name {vnet} --subnet {subnet}')

        # prepare workspace
        workspace = self.cmd('monitor log-analytics workspace create '
                             '--resource-group {rg} '
                             '--location eastus '
                             '--workspace-name {workspace} ').get_output_in_json()
        self.kwargs.update({
            'workspace_id': workspace['id']
        })

        self.cmd('network watcher flow-log create '
                 '--location {location} '
                 '--resource-group {rg} '
                 '--vnet {vnet} '
                 '--storage-account {storage_account} '
                 '--workspace {workspace_id} '
                 '--name {flow_log} ')

        res1 = self.cmd('network watcher flow-log show --location {location} --name {flow_log}').get_output_in_json()
        self.assertEqual(res1['name'], self.kwargs['flow_log'])
        self.assertEqual(res1['enabled'], True)
        self.assertEqual(res1['retentionPolicy']['days'], 0)
        self.assertEqual(res1['retentionPolicy']['enabled'], False)
        # self.assertIsNone(res1['tags'])

        #update targetId from vnet to nic
        res2 = self.cmd('network watcher flow-log update '
                        '--location {location} '
                        '--name {flow_log} '
                        '--nic {nic} '
                        '--resource-group {rg} '
                        '--retention 2 '
                        '--tags foo=bar ').get_output_in_json()
        self.assertEqual(res2['name'], self.kwargs['flow_log'])
        self.assertEqual(res2['enabled'], True)
        self.assertTrue(res2['targetResourceId'].endswith(self.kwargs['nic']))
        self.assertEqual(res2['name'], self.kwargs['flow_log'])
        self.assertEqual(res2['retentionPolicy']['days'], 2)
        self.assertEqual(res2['retentionPolicy']['enabled'], True)
        # self.assertIsNotNone(res2['tags'])
        
        self.cmd('network watcher flow-log delete --location {location} --name {flow_log}')

        with self.assertRaisesRegex(SystemExit, '3'):
            self.cmd('network watcher flow-log show --location {location} --name {flow_log}')

        #targetId as subnet
        self.cmd('network watcher flow-log create '
                 '--location {location} '
                 '--resource-group {rg} '
                 '--vnet {vnet} '
                 '--subnet {subnet} '
                 '--storage-account {storage_account} '
                 '--workspace {workspace_id} '
                 '--name {flow_log} ')

        res1 = self.cmd('network watcher flow-log show --location {location} --name {flow_log}').get_output_in_json()
        self.assertEqual(res1['name'], self.kwargs['flow_log'])
        self.assertEqual(res1['enabled'], True)
        self.assertEqual(res1['retentionPolicy']['days'], 0)
        self.assertEqual(res1['retentionPolicy']['enabled'], False)
        # self.assertIsNone(res1['tags'])

        #update targetId from subnet to nsg
        res2 = self.cmd('network watcher flow-log update '
                        '--location {location} '
                        '--name {flow_log} '
                        '--nsg {nsg} '
                        '--resource-group {rg} '
                        '--retention 2 '
                        '--tags foo=bar ').get_output_in_json()
        self.assertEqual(res2['name'], self.kwargs['flow_log'])
        self.assertEqual(res2['enabled'], True)
        self.assertTrue(res2['targetResourceId'].endswith(self.kwargs['nsg']))
        self.assertEqual(res2['name'], self.kwargs['flow_log'])
        self.assertEqual(res2['retentionPolicy']['days'], 2)
        self.assertEqual(res2['retentionPolicy']['enabled'], True)
        self.assertIsNotNone(res2['tags'])

        self.cmd('network watcher flow-log delete --location {location} --name {flow_log}')

        with self.assertRaisesRegex(SystemExit, '3'):
            self.cmd('network watcher flow-log show --location {location} --name {flow_log}')

        #targetId as NSG
        self.cmd('network watcher flow-log create '
                 '--location {location} '
                 '--resource-group {rg} '
                 '--nsg {nsg} '
                 '--storage-account {storage_account} '
                 '--workspace {workspace_id} '
                 '--name {flow_log} ')

        res1 = self.cmd('network watcher flow-log show --location {location} --name {flow_log}').get_output_in_json()
        self.assertEqual(res1['name'], self.kwargs['flow_log'])
        self.assertEqual(res1['enabled'], True)
        self.assertEqual(res1['retentionPolicy']['days'], 0)
        self.assertEqual(res1['retentionPolicy']['enabled'], False)
        # self.assertIsNone(res1['tags'])

        #update targetId from nsg to vnet
        res2 = self.cmd('network watcher flow-log update '
                        '--location {location} '
                        '--name {flow_log} '
                        '--vnet {vnet} '
                        '--resource-group {rg} '
                        '--retention 2 '
                        '--tags foo=bar ').get_output_in_json()
        self.assertEqual(res2['name'], self.kwargs['flow_log'])
        self.assertEqual(res2['enabled'], True)
        self.assertTrue(res2['targetResourceId'].endswith(self.kwargs['vnet']))
        self.assertEqual(res2['name'], self.kwargs['flow_log'])
        self.assertEqual(res2['retentionPolicy']['days'], 2)
        self.assertEqual(res2['retentionPolicy']['enabled'], True)
        self.assertIsNotNone(res2['tags'])
        
    @ResourceGroupPreparer(name_prefix='test_nw_flow_log_', location='eastus')
    @StorageAccountPreparer(name_prefix='testflowlog', location='eastus', kind='StorageV2')
    def test_nw_flow_log_update(self, resource_group, resource_group_location, storage_account):
        self.kwargs.update({
            'rg': resource_group,
            'location': resource_group_location,
            'storage_account': storage_account,
            'storage_account_2': 'storageaccount0395',
            'nsg': 'nsg1',
            'watcher_rg': 'NetworkWatcherRG',
            'watcher_name': 'NetworkWatcher_{}'.format(resource_group_location),
            'flow_log': 'flow_log_test2',
            'workspace': self.create_random_name('clitest', 20),
        })

        # enable network watcher
        # self.cmd('network watcher configure -g {rg} --locations {location} --enabled')

        # prepare the target resource
        nsg_info = self.cmd('network nsg create -g {rg} -n {nsg}').get_output_in_json()
        self.kwargs.update({
            'nsg_id': nsg_info['NewNSG']['id']
        })

        # prepare another storage account in another resource group
        storage_info = self.cmd('storage account create '
                                '--resource-group {rg} '
                                '--name {storage_account_2} --https-only').get_output_in_json()
        self.kwargs.update({
            'another_storage': storage_info['id']
        })

        # prepare workspace
        workspace = self.cmd('monitor log-analytics workspace create '
                             '--resource-group {rg} '
                             '--location {location} '
                             '--workspace-name {workspace} ').get_output_in_json()
        self.kwargs.update({
            'workspace_id': workspace['id']
        })

        self.cmd('network watcher flow-log create '
                 '--location {location} '
                 '--resource-group {rg} '
                 '--nsg {nsg} '
                 '--storage-account {storage_account} '
                 '--workspace {workspace_id} '
                 '--name {flow_log} ')

        res1 = self.cmd('network watcher flow-log show --location {location} --name {flow_log}').get_output_in_json()
        self.assertEqual(res1['name'], self.kwargs['flow_log'])
        self.assertEqual(res1['enabled'], True)
        self.assertEqual(res1['retentionPolicy']['days'], 0)
        self.assertEqual(res1['retentionPolicy']['enabled'], False)
        self.assertTrue(res1['storageId'].endswith(self.kwargs['storage_account']))
        # self.assertIsNone(res1['tags'])

        res2 = self.cmd('network watcher flow-log update '
                        '--location {location} '
                        '--name {flow_log} '
                        '--retention 2 '
                        '--storage-account {another_storage} '
                        '--tags foo=bar ').get_output_in_json()
        self.assertEqual(res2['name'], self.kwargs['flow_log'])
        self.assertEqual(res2['enabled'], True)
        self.assertEqual(res2['retentionPolicy']['days'], 2)
        self.assertEqual(res2['retentionPolicy']['enabled'], True)
        self.assertTrue(res2['storageId'].endswith(self.kwargs['storage_account_2']))
        self.assertIsNotNone(res2['tags'])
